package positron;

import java.io.File;
import java.io.IOException;

/**
 * Call acceptance tests from the host OS.
 *
 * Usage: java positron.RunTests my.android.app [.Positron] [/path/to/android_sdk]
 *
 * The quick version: adb shell "/system/bin/am instrument -w my.android.app/.Positron"
 *
 * RunTests allows users to run their tests from a script or directly from
 * eclipse as a 'Java Application' target. It expects one to three parameters.
 *
 * The first is the tested package (the same as the targetPackage attribute you
 * declared the Positron instrumentation with.)  It is mandatory.
 *
 * The second parameter is optional and names the instrumentation to run.  It
 * corrosponds to the class attribute on the instrumentation tag.  The default
 * is ".Positron".  If you named your positron subclass differently, supply
 * that name here.  Do no forget the leading period.
 *
 * The third parameter is optional and declares where to find the android sdk
 * installation.  It should be the absolute path to the parent of the standard
 * 'tools' directory.  If omitted, the value is taken from the ANDROID_HOME
 * environment variable, which is then mandatory.
 *
 * @author philhsmith
 */
public class RunTests {
	public static void main(String [] args) throws IOException {
		if (args.length == 0) {
			System.err.println("Usage: java positron.RunTests my.android.app [.Positron] [/path/to/android_sdk]");
			System.err.println();
			System.err.println("The quick version: adb shell \"/system/bin/am instrument -w my.android.app/.Positron\"");
			System.err.println();
			System.err.println("RunTests allows users to run their tests from a script or directly from");
			System.err.println("eclipse as a 'Java Application' target. It expects one to three parameters.");
			System.err.println();
			System.err.println("The first is the tested package (the same as the targetPackage attribute you");
			System.err.println("declared the Positron instrumentation with.)  It is mandatory.");
			System.err.println();
			System.err.println("The second parameter is optional and names the instrumentation to run.  It");
			System.err.println("corrosponds to the class attribute on the instrumentation tag.  The default");
			System.err.println("is \".Positron\".  If you named your positron subclass differently, supply");
			System.err.println("that name here.  Do no forget the leading period.");
			System.err.println();
			System.err.println("The third parameter is optional and declares where to find the android sdk");
			System.err.println("installation.  It should be the absolute path to the parent of the standard");
			System.err.println("'tools' directory.  If omitted, the value is taken from the ANDROID_HOME");
			System.err.println("environment variable, which is then mandatory.");
			return;
		}

		String targetPackage = args[0];
		String className = args.length >= 2 ? args[1] : ".Positron";
		String androidHomePath = args.length >= 3 ? args[2] : System.getenv("ANDROID_HOME");

		if (androidHomePath == null) {
			System.err.println("Please pass the location of your" +
					" android sdk.  Do so by either passing it as the third parameter," +
					" or by setting the ANDROID_HOME environment variable.  For example," +
					" you may copy the location from the Android preferences pane in eclipse.");
			return;
		}

		File androidHome = new File(androidHomePath);
		if (!androidHome.exists()) throw new IllegalArgumentException(androidHomePath + " doesn't exist.");
		if (!androidHome.isDirectory()) throw new IllegalArgumentException(androidHomePath + " is not a directory!");

		File adb = null;
		for (File program : new File(androidHome, "tools").listFiles()) {
			if (!program.getName().startsWith("adb")) continue;

			adb = program;
			break;
		}

		if (adb == null) throw new IllegalArgumentException("Can't find adb in " + androidHomePath + "/tools?!");

		String adbPath = adb.getCanonicalPath();
		String passedCommand = "/system/bin/am instrument -w " + targetPackage + "/" + className;

		Runtime.getRuntime().exec(new String [] { adbPath, "shell", passedCommand });
	}
}
