package positron.harness;

import static java.lang.reflect.Modifier.isStatic;
import static positron.harness.IOUtils.basename;
import static positron.harness.IOUtils.cp;
import static positron.harness.IOUtils.ls;
import static positron.harness.IOUtils.mv;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.lang.reflect.Field;

import positron.Positron;
import android.app.Instrumentation;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.util.Log;

public class DatabaseFixture {
	private static final String TAG = Positron.TAG;

	private final Instrumentation instrumentation;

	public DatabaseFixture(Instrumentation instrumentation) {
		this.instrumentation = instrumentation;
	}

	/** Backup all databases in the target context. */
	public void backup() {
		// by DTM
		String dir = databaseDir();
		if ( dir != null ) {
			for (String database : ls(dir, ".*\\.db")) {
				database = basename(database);
				if (isBackup(database)) continue;
				backup(database);
			}
		}
	}

	/** Restore all databases that have backups in the target context. */
	public void restore() {
		// by DTM
		String dir = databaseDir();
		if ( dir != null ) { 
			for (String backup : ls(dir, ".*_backup\\.db")) {
				String database = basename(backup).replaceFirst("_backup", "");
				restore(database);
			}
		}
	}

	/**
	 * Back up the given database by copying its defining file out of the way.
	 *
	 * This blows away any previous backup.
	 *
	 * @param database The database to back up.
	 */
	public void backup(String database) {
		Log.i(TAG, "Backing up " + database);
		cp(locationOf(database), locationOf(backupOf(database)));
	}

	/**
	 * Restore the given database by copying the backup file back in place.
	 *
	 * @param database The database to restore.
	 */
	public void restore(String database) {
		Log.i(TAG, "Restoring " + database);
		mv(locationOf(backupOf(database)), locationOf(database));
	}

	private String locationOf(String database) {
		return databaseDir() + "/" + database;
	}

	private String databaseDir() {
		// by DTM
		// perhaps there's no database dir in this application
		String dir = "/data/data/" + instrumentation.getTargetContext().getPackageName() + "/databases";
		if ( ! new File(dir).isDirectory() ) {
			// there's no database dir
			return null;
		}
		
		return dir;
	}

	private String backupOf(String database) {
		return database.replaceAll("\\.db$", "_backup.db");
	}

	private boolean isBackup(String database) {
		return database.matches(".*_backup\\.db$");
	}

	/**
	 * Execute Sql scripts loaded from a raw resources.
	 *
	 * @param database The database to execute the sql.
	 * @param scriptsAsRawResources A varargs of R.raw.XXX resource ids of scripts to execute.
	 */
	public void execSql(String database, int... scriptsAsRawResources) {
		SQLiteDatabase db = openDatabase(database);
		SqlScanner statements = null;

		for (int id : scriptsAsRawResources) {
			try {
				statements = new SqlScanner(openRawResource(id));
				for (String statement : statements) db.execSQL(statement);
			} catch(SQLiteException e) {
				String script = getRawResourceName(id);
				int line = statements.getLineNumber();
				throw new RuntimeException("\nIn " + script + ", line " + line + ":\n" + e.getMessage(), e);
			} finally {
				if (statements != null) statements.close();
			}
		}
	}

	private SQLiteDatabase openDatabase(String name) {
		try {
			return instrumentation.getTargetContext().openDatabase(name, null);
		} catch (FileNotFoundException e) {
			throw new IllegalArgumentException("Database " + name + " wasn't found.", e);
		}
	}

	private InputStream openRawResource(int id) {
		return instrumentation.getTargetContext().getResources().openRawResource(id);
	}

	private String getRawResourceName(int id) {
		String targetPackage = instrumentation.getTargetContext().getPackageName();
		try {
			Class<?> rRaw = Class.forName(targetPackage + ".R$raw");
			for (Field resource : rRaw.getFields()) {
				if (!isStatic(resource.getModifiers()) || resource.getType() != Integer.TYPE) continue;
				if (resource.getInt(null) != id) continue;

				return "R.raw." + resource.getName();
			}

			return "Unknown";
		} catch (ClassNotFoundException e) {
			Log.e(TAG, "Can't find " + targetPackage + ".R.raw.  Are your sql fixture scripts declared as raw resources?");
			throw new RuntimeException(e.getMessage(), e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e.getMessage(), e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}
}
