package positron.harness;

import static java.lang.Integer.parseInt;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import android.app.Activity;
import android.view.ViewGroup;

/**
 * Use dot notation to navigate getters and array/child indexing.
 * 
 * All access is done through method reflection; no public properties are
 * examined.
 * 
 * Getter getXxx may be specified as ".getXxx" or ".xxx".  Note that getters
 * that don't start with "get" just use their names: for String.length() use
 * "length".
 * 
 * Arrays and ViewGroups may be indexed.  To index a ViewGroup, getChildAt is
 * called.  Indexes are specified with a dot: ".listView.2" on a ListActivity
 * would yield getListView().getChildAt(2).
 * 
 * Use a # prefix to do a child view lookup by id.  This is only legal when the
 * current receiving object is an Activity (though this could be extended; make
 * some noise or patch if you really want it.)  The ids themselves should be the
 * id strings just as they appear in your layout xml.
 * 
 * So "#my_text_view.text" would findViewById(R.id.my_text_view).getText().
 * 
 * Clearly all this reflection bypasses type checking; you will get
 * ClassCastExceptions, etc, if you are not careful.
 * 
 * @author philhsmith
 */
public class ViewShorthand {
	@SuppressWarnings("unchecked")
	public <T> T evaluate(Class<T> type, Object target, String query) {
		return (T)evaluate(target, query);
	}
	
	public Object evaluate(Object target, String query) {
		if (query == null || query.length() == 0) return target;
		for (String term : query.split("\\.")) target = evaluateTerm(target, term);		
		return target;
	}
	
	private Object evaluateTerm(Object target, String term) {
		try {
			return index(target, parseInt(term));
		} catch(NumberFormatException notAnInt) {}

		if (term.startsWith("#")) return getById(target, term.substring(1));
		
		return get(target, term);
	}
	
	private Object index(Object target, int position) {
		if (target.getClass().isArray()) return Array.get(target, position);
		if (target instanceof ViewGroup) return ((ViewGroup)target).getChildAt(position);
		
		throw new IllegalArgumentException("Don't know how to index into a " + target.getClass().getSimpleName() + ".");
	}
	
	private Object getById(Object target, String id) {
		if (!(target instanceof Activity)) {
			/*
			 * If there's demand for more sophisticated id searching, this could recursively walk ViewGroups while
			 * calling getId, looking for matches.  I'm not going to bother yet, since it doesn't seem worth it.
			 */
			throw new IllegalArgumentException("" + target + " isn't an Activity, so we can't call findViewById.");
		}
				
		Activity activity = (Activity)target;
		String pkg = activity.getPackageName();
		
		try {
			Class<?> R_id = activity.getClassLoader().loadClass(pkg + ".R$id");
			int actualId = R_id.getField(id).getInt(null);
			return activity.findViewById(actualId);
		} catch (ClassNotFoundException e) {
			throw new IllegalArgumentException("Can't find class " + pkg + ".R.id!?!");
		} catch (NoSuchFieldException e) {
			throw new IllegalArgumentException("Unknown id " + pkg + ".R.id." + id + ".  Spelling?");
		} catch (SecurityException e) { throw new RuntimeException(e.getMessage(), e);
		} catch (IllegalAccessException e) { throw new RuntimeException(e.getMessage(), e); }
	}
	
	private Object get(Object target, String property) {
		String [] getterNames = new String [] {
				property,
				"get" + property.substring(0, 1).toUpperCase() + property.substring(1)
		};
		
		Method getter = null;
		for (String getterName : getterNames) {
			try {
				getter = target.getClass().getMethod(getterName);
				break;
			} catch (SecurityException e) { throw new RuntimeException(e.getMessage(), e);
			} catch (NoSuchMethodException e) { /* try another. */ }			
		}
		
		if (getter == null) throw new IllegalArgumentException("Can't find " + property + " on " + target + ".");
			
		try {
			return getter.invoke(target);
		} catch (IllegalArgumentException e) { throw new RuntimeException(e.getMessage(), e);
		} catch (IllegalAccessException e) { throw new RuntimeException(e.getMessage(), e);
		} catch (InvocationTargetException e) { throw new RuntimeException(e.getMessage(), e); }
	}	
}
